@@ -22,6 +22,8 @@ type Var struct {
 	Name string
 	// Len is the length of the replaced variable.
 	Len int
+	// Defined indicates whether the variable has been defined.
+	Defined bool
 }
 
 // String satisfies the fmt.Stringer interface.
@@ -99,7 +101,7 @@ func (b *Stmt) RawString() string {
 	var i int
 	// deinterpolate vars
 	for _, v := range b.Vars {
-		if v.Len == 0 {
+		if !v.Defined {
 			continue
 		}
 		if len(s) > i {
@@ -244,6 +246,7 @@ parse:
 				}
 				b.Vars = append(b.Vars, v)
 				if ok, z, _ := unquote(q+v.Name+q, true); ok {
+					v.Defined = true
 					b.r, b.rlen = substituteVar(b.r, v, z)
 					i--
 				}
